@@ -133,15 +133,19 @@ static void print_explain(FILE *f)
 		"                [ min_links MIN_LINKS ]\n"
 		"                [ lp_interval LP_INTERVAL ]\n"
 		"                [ packets_per_slave PACKETS_PER_SLAVE ]\n"
+		"		 [ tlb_dynamic_lb TLB_DYNAMIC_LB ]\n"
 		"                [ lacp_rate LACP_RATE ]\n"
 		"                [ ad_select AD_SELECT ]\n"
+		"                [ ad_user_port_key PORTKEY ]\n"
+		"                [ ad_actor_sys_prio SYSPRIO ]\n"
+		"                [ ad_actor_system LLADDR ]\n"
 		"\n"
 		"BONDMODE := balance-rr|active-backup|balance-xor|broadcast|802.3ad|balance-tlb|balance-alb\n"
 		"ARP_VALIDATE := none|active|backup|all\n"
 		"ARP_ALL_TARGETS := any|all\n"
 		"PRIMARY_RESELECT := always|better|failure\n"
 		"FAIL_OVER_MAC := none|active|follow\n"
-		"XMIT_HASH_POLICY := layer2|layer2+3|layer3+4\n"
+		"XMIT_HASH_POLICY := layer2|layer2+3|layer3+4|encap2+3|encap3+4\n"
 		"LACP_RATE := slow|fast\n"
 		"AD_SELECT := stable|bandwidth|count\n"
 	);
@@ -157,7 +161,8 @@ static int bond_parse_opt(struct link_util *lu, int argc, char **argv,
 {
 	__u8 mode, use_carrier, primary_reselect, fail_over_mac;
 	__u8 xmit_hash_policy, num_peer_notif, all_slaves_active;
-	__u8 lacp_rate, ad_select;
+	__u8 lacp_rate, ad_select, tlb_dynamic_lb;
+	__u16 ad_user_port_key, ad_actor_sys_prio;
 	__u32 miimon, updelay, downdelay, arp_interval, arp_validate;
 	__u32 arp_all_targets, resend_igmp, min_links, lp_interval;
 	__u32 packets_per_slave;
@@ -166,10 +171,8 @@ static int bond_parse_opt(struct link_util *lu, int argc, char **argv,
 	while (argc > 0) {
 		if (matches(*argv, "mode") == 0) {
 			NEXT_ARG();
-			if (get_index(mode_tbl, *argv) < 0) {
+			if (get_index(mode_tbl, *argv) < 0)
 				invarg("invalid mode", *argv);
-				return -1;
-			}
 			mode = get_index(mode_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_MODE, mode);
 		} else if (matches(*argv, "active_slave") == 0) {
@@ -182,38 +185,28 @@ static int bond_parse_opt(struct link_util *lu, int argc, char **argv,
 			addattr32(n, 1024, IFLA_BOND_ACTIVE_SLAVE, 0);
 		} else if (matches(*argv, "miimon") == 0) {
 			NEXT_ARG();
-			if (get_u32(&miimon, *argv, 0)) {
+			if (get_u32(&miimon, *argv, 0))
 				invarg("invalid miimon", *argv);
-				return -1;
-			}
 			addattr32(n, 1024, IFLA_BOND_MIIMON, miimon);
 		} else if (matches(*argv, "updelay") == 0) {
 			NEXT_ARG();
-			if (get_u32(&updelay, *argv, 0)) {
+			if (get_u32(&updelay, *argv, 0))
 				invarg("invalid updelay", *argv);
-				return -1;
-			}
 			addattr32(n, 1024, IFLA_BOND_UPDELAY, updelay);
 		} else if (matches(*argv, "downdelay") == 0) {
 			NEXT_ARG();
-			if (get_u32(&downdelay, *argv, 0)) {
+			if (get_u32(&downdelay, *argv, 0))
 				invarg("invalid downdelay", *argv);
-				return -1;
-			}
 			addattr32(n, 1024, IFLA_BOND_DOWNDELAY, downdelay);
 		} else if (matches(*argv, "use_carrier") == 0) {
 			NEXT_ARG();
-			if (get_u8(&use_carrier, *argv, 0)) {
+			if (get_u8(&use_carrier, *argv, 0))
 				invarg("invalid use_carrier", *argv);
-				return -1;
-			}
 			addattr8(n, 1024, IFLA_BOND_USE_CARRIER, use_carrier);
 		} else if (matches(*argv, "arp_interval") == 0) {
 			NEXT_ARG();
-			if (get_u32(&arp_interval, *argv, 0)) {
+			if (get_u32(&arp_interval, *argv, 0))
 				invarg("invalid arp_interval", *argv);
-				return -1;
-			}
 			addattr32(n, 1024, IFLA_BOND_ARP_INTERVAL, arp_interval);
 		} else if (matches(*argv, "arp_ip_target") == 0) {
 			struct rtattr * nest = addattr_nest(n, 1024,
@@ -234,18 +227,14 @@ static int bond_parse_opt(struct link_util *lu, int argc, char **argv,
 			addattr_nest_end(n, nest);
 		} else if (matches(*argv, "arp_validate") == 0) {
 			NEXT_ARG();
-			if (get_index(arp_validate_tbl, *argv) < 0) {
+			if (get_index(arp_validate_tbl, *argv) < 0)
 				invarg("invalid arp_validate", *argv);
-				return -1;
-			}
 			arp_validate = get_index(arp_validate_tbl, *argv);
 			addattr32(n, 1024, IFLA_BOND_ARP_VALIDATE, arp_validate);
 		} else if (matches(*argv, "arp_all_targets") == 0) {
 			NEXT_ARG();
-			if (get_index(arp_all_targets_tbl, *argv) < 0) {
+			if (get_index(arp_all_targets_tbl, *argv) < 0)
 				invarg("invalid arp_all_targets", *argv);
-				return -1;
-			}
 			arp_all_targets = get_index(arp_all_targets_tbl, *argv);
 			addattr32(n, 1024, IFLA_BOND_ARP_ALL_TARGETS, arp_all_targets);
 		} else if (matches(*argv, "primary") == 0) {
@@ -256,94 +245,113 @@ static int bond_parse_opt(struct link_util *lu, int argc, char **argv,
 			addattr32(n, 1024, IFLA_BOND_PRIMARY, ifindex);
 		} else if (matches(*argv, "primary_reselect") == 0) {
 			NEXT_ARG();
-			if (get_index(primary_reselect_tbl, *argv) < 0) {
+			if (get_index(primary_reselect_tbl, *argv) < 0)
 				invarg("invalid primary_reselect", *argv);
-				return -1;
-			}
 			primary_reselect = get_index(primary_reselect_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_PRIMARY_RESELECT,
 				 primary_reselect);
 		} else if (matches(*argv, "fail_over_mac") == 0) {
 			NEXT_ARG();
-			if (get_index(fail_over_mac_tbl, *argv) < 0) {
+			if (get_index(fail_over_mac_tbl, *argv) < 0)
 				invarg("invalid fail_over_mac", *argv);
-				return -1;
-			}
 			fail_over_mac = get_index(fail_over_mac_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_FAIL_OVER_MAC,
 				 fail_over_mac);
 		} else if (matches(*argv, "xmit_hash_policy") == 0) {
 			NEXT_ARG();
-			if (get_index(xmit_hash_policy_tbl, *argv) < 0) {
+			if (get_index(xmit_hash_policy_tbl, *argv) < 0)
 				invarg("invalid xmit_hash_policy", *argv);
-				return -1;
-			}
+
 			xmit_hash_policy = get_index(xmit_hash_policy_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_XMIT_HASH_POLICY,
 				 xmit_hash_policy);
 		} else if (matches(*argv, "resend_igmp") == 0) {
 			NEXT_ARG();
-			if (get_u32(&resend_igmp, *argv, 0)) {
+			if (get_u32(&resend_igmp, *argv, 0))
 				invarg("invalid resend_igmp", *argv);
-				return -1;
-			}
+
 			addattr32(n, 1024, IFLA_BOND_RESEND_IGMP, resend_igmp);
 		} else if (matches(*argv, "num_grat_arp") == 0 ||
 			   matches(*argv, "num_unsol_na") == 0) {
 			NEXT_ARG();
-			if (get_u8(&num_peer_notif, *argv, 0)) {
+			if (get_u8(&num_peer_notif, *argv, 0))
 				invarg("invalid num_grat_arp|num_unsol_na",
 				       *argv);
-				return -1;
-			}
+
 			addattr8(n, 1024, IFLA_BOND_NUM_PEER_NOTIF,
 				 num_peer_notif);
 		} else if (matches(*argv, "all_slaves_active") == 0) {
 			NEXT_ARG();
-			if (get_u8(&all_slaves_active, *argv, 0)) {
+			if (get_u8(&all_slaves_active, *argv, 0))
 				invarg("invalid all_slaves_active", *argv);
-				return -1;
-			}
+
 			addattr8(n, 1024, IFLA_BOND_ALL_SLAVES_ACTIVE,
 				 all_slaves_active);
 		} else if (matches(*argv, "min_links") == 0) {
 			NEXT_ARG();
-			if (get_u32(&min_links, *argv, 0)) {
+			if (get_u32(&min_links, *argv, 0))
 				invarg("invalid min_links", *argv);
-				return -1;
-			}
+
 			addattr32(n, 1024, IFLA_BOND_MIN_LINKS, min_links);
 		} else if (matches(*argv, "lp_interval") == 0) {
 			NEXT_ARG();
-			if (get_u32(&lp_interval, *argv, 0)) {
+			if (get_u32(&lp_interval, *argv, 0))
 				invarg("invalid lp_interval", *argv);
-				return -1;
-			}
+
 			addattr32(n, 1024, IFLA_BOND_LP_INTERVAL, lp_interval);
 		} else if (matches(*argv, "packets_per_slave") == 0) {
 			NEXT_ARG();
-			if (get_u32(&packets_per_slave, *argv, 0)) {
+			if (get_u32(&packets_per_slave, *argv, 0))
 				invarg("invalid packets_per_slave", *argv);
-				return -1;
-			}
+
 			addattr32(n, 1024, IFLA_BOND_PACKETS_PER_SLAVE,
 				  packets_per_slave);
 		} else if (matches(*argv, "lacp_rate") == 0) {
 			NEXT_ARG();
-			if (get_index(lacp_rate_tbl, *argv) < 0) {
+			if (get_index(lacp_rate_tbl, *argv) < 0)
 				invarg("invalid lacp_rate", *argv);
-				return -1;
-			}
+
 			lacp_rate = get_index(lacp_rate_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_AD_LACP_RATE, lacp_rate);
 		} else if (matches(*argv, "ad_select") == 0) {
 			NEXT_ARG();
-			if (get_index(ad_select_tbl, *argv) < 0) {
+			if (get_index(ad_select_tbl, *argv) < 0)
 				invarg("invalid ad_select", *argv);
-				return -1;
-			}
+
 			ad_select = get_index(ad_select_tbl, *argv);
 			addattr8(n, 1024, IFLA_BOND_AD_SELECT, ad_select);
+		} else if (matches(*argv, "ad_user_port_key") == 0) {
+			NEXT_ARG();
+			if (get_u16(&ad_user_port_key, *argv, 0))
+				invarg("invalid ad_user_port_key", *argv);
+
+			addattr16(n, 1024, IFLA_BOND_AD_USER_PORT_KEY,
+				  ad_user_port_key);
+		} else if (matches(*argv, "ad_actor_sys_prio") == 0) {
+			NEXT_ARG();
+			if (get_u16(&ad_actor_sys_prio, *argv, 0))
+				invarg("invalid ad_actor_sys_prio", *argv);
+
+			addattr16(n, 1024, IFLA_BOND_AD_ACTOR_SYS_PRIO,
+				  ad_actor_sys_prio);
+		} else if (matches(*argv, "ad_actor_system") == 0) {
+			int len;
+			char abuf[32];
+
+			NEXT_ARG();
+			len = ll_addr_a2n(abuf, sizeof(abuf), *argv);
+			if (len < 0)
+				return -1;
+			addattr_l(n, 1024, IFLA_BOND_AD_ACTOR_SYSTEM,
+				  abuf, len);
+		} else if (matches(*argv, "tlb_dynamic_lb") == 0) {
+			NEXT_ARG();
+			if (get_u8(&tlb_dynamic_lb, *argv, 0)) {
+				invarg("invalid tlb_dynamic_lb", *argv);
+				return -1;
+			}
+			addattr8(n, 1024, IFLA_BOND_TLB_DYNAMIC_LB,
+				 tlb_dynamic_lb);
 		} else if (matches(*argv, "help") == 0) {
 			explain();
 			return -1;
@@ -534,6 +542,30 @@ static void bond_print_opt(struct link_util *lu, FILE *f, struct rtattr *tb[])
 				ll_addr_n2a(p, ETH_ALEN, 0, b, sizeof(b)));
 		}
 	}
+
+	if (tb[IFLA_BOND_AD_ACTOR_SYS_PRIO]) {
+		fprintf(f, "ad_actor_sys_prio %u ",
+			rta_getattr_u16(tb[IFLA_BOND_AD_ACTOR_SYS_PRIO]));
+	}
+
+	if (tb[IFLA_BOND_AD_USER_PORT_KEY]) {
+		fprintf(f, "ad_user_port_key %u ",
+			rta_getattr_u16(tb[IFLA_BOND_AD_USER_PORT_KEY]));
+	}
+
+	if (tb[IFLA_BOND_AD_ACTOR_SYSTEM]) {
+		/* We assume the l2 address is an Ethernet MAC address */
+		SPRINT_BUF(b1);
+		fprintf(f, "ad_actor_system %s ",
+			ll_addr_n2a(RTA_DATA(tb[IFLA_BOND_AD_ACTOR_SYSTEM]),
+				    RTA_PAYLOAD(tb[IFLA_BOND_AD_ACTOR_SYSTEM]),
+				    1 /*ARPHDR_ETHER*/, b1, sizeof(b1)));
+	}
+
+	if (tb[IFLA_BOND_TLB_DYNAMIC_LB]) {
+		fprintf(f, "tlb_dynamic_lb %u ",
+			rta_getattr_u8(tb[IFLA_BOND_TLB_DYNAMIC_LB]));
+	}
 }
 
 static void bond_print_help(struct link_util *lu, int argc, char **argv,
